@@ -8,8 +8,7 @@ use NEXT;
 
 use Data::Dumper;
 
-
-our ($VERSION) = ('$Revision: 2.0 $' =~ m/([\.\d]+)/) ;
+our $VERSION = '2.1_01';
 
 
 =head1 NAME
@@ -63,7 +62,7 @@ in the end() method, for example, to automatically forward all actions
 to the Seamstress view class.
 
     # In MyApp or MyApp::Controller::SomeController
-    
+
     sub end : Private {
         my( $self, $c ) = @_;
         $c->forward('MyApp::View::Seamstress');
@@ -72,6 +71,9 @@ to the Seamstress view class.
 Or you might like to use 
 L<Catalyst::Plugin::DefaultEnd|Catalyst::Plugin::DefaultEnd>
 
+or even
+L<Catalyst::Action::RenderView|Catalyst::Action::RenderView>
+
 
 =head1 CONFIGURATION
 
@@ -80,54 +82,11 @@ configuration info in C<MyApp::View::Seamstress>. You configure the
 per-request information (e.g. C<< $c->stash->{LOOM} >> and
 variables for this template) in your controller.
 
+The two main options which control how View::Seamtress renders HTML are the
+LOOM (which is taken from the stash) and optionally the skeleton, which is
+stored in the app config.
 
-=head2 The meat-skeleton paradigm
-
-When Catalyst::View::Seamstress is forwarded to, it will operate in one of 
-2 ways: a plain meat way or a meat-skeleton way. Plain meat is simple:
-the View takes C<< $c->stash->{template} >> and calls C<new()> and
-C<process()> on it and stores the result in 
-C<< $c->response->body >>. Meat-skeleton is designed to facilitate the
-way that web sites are typically designed. It's discussion follows:
-
-HTML pages typically have meat and a skeleton. The meat varies from page
-to page while the skeleton is fairly (though not completely) 
-static. For example, the skeleton of a webpage is usually a header, a
-footer, and a navbar. The meat is what shows up when you click on a
-link on the page somewhere. While the meat will change with each
-click, the skeleton is rather static.
-
-The perfect example of 
-
-Mason accomodates the meat-skeleton paradigm via
-an C<autohandler> and C<< $m->call_next() >>. Template 
-accomodates it via its C<WRAPPER> directive.
-
-And Seamstress? Well, here's what you _can_ do:
-
-=over
-
-=item 1 generate the meat, C<$meat>
-
-This is typically what you see in the C<body> part of an HTML page
-
-=item 2 generate the skeleton, C<$skeleton>
-
-This is typically the html, head, and maybe some body 
-
-=item 3 put the meat in the skeleton
-
-=back
-
-So, nothing about this is forced. This is just how I typically do
-things and that is why
-L<Catalyst::View::Seamstress|Catalyst::View::Seamstress> has support
-for this.
-
-
-=head3 Choosing plain-meat or meat-skeleton 
-
-There are two items which control how this plugin renders HTML.
+If you just configure a LOOM then you are most likely using the "plain meat" method described below. If you also configure a skeleton in your config as well then you're using the "meat and skeleton" method. See below for a more detailed discussion of this!
 
 =over
 
@@ -160,21 +119,55 @@ and C<< $stash->{LOOM}->fixup() >>
 come into effect
 as described in L<HTML::Seamstress/"The_meat-skeleton_paradigm">.
 
-Let's take that a little slower: C<< $stash->{template}->fixup() >>
+Let's take that a little slower: C<< $stash->{LOOM}->fixup() >>
 means: given a Seamstress-style Perl class, whose name is
-C<< $stash->{template} >>, call the method C<fixup()> in that
+C<< $stash->{LOOM} >>, call the method C<fixup()> in that
 class so that it can do a final fixup of the entire HTML that is about
 to be shipped back to the client.
 
+=back
+
+The output generated from the LOOM
+(and possibly its interaction with a skeleton)
+is stored in
+C<< $c->response->body >>.
+
+
+=head2 Other Config Options
+
+=over
+
+=item config->{fixup}
+
+Set this to a coderef to allow the view to change the tree after the main
+processing phase. 
+
+=item config->{use_xhtml}
+
+By default the view will generate html 4 style html by calling as_HTML on the
+tree object. If you set this to a true value it will generate XHTML style HTML
+by calling as_XML on the tree object. See L<HTML::Element> for details for
+these methods.
+
+Also note that this won't apply proper HTML doctypes and what-have-you unless
+you have them in your original HTML.
+
+=item config->{meat_pack}
+
+This is the subref which is called to pack meat into the skeleton for the meat
+skeleton method. Tinker with this to have more creative LOOMS. See "Funny
+LOOMs" and the meat/skeleton discussions.
 
 =back
 
+=head2 Funny LOOMs
 
+In the examples so far the LOOM has always been a class name.
 
-The output generated by the template 
-(and possibly its interaction with a skeleton) 
-is stored in
-C<< $c->response->body >>.
+If instead LOOM is an object then we'll assume that is a useful HTML::Element style
+object and just use that instead of calling C<new> on the LOOM. In this case we'll also not ->delete it at the end of the request so you'll have to do that yourself!
+
+If the LOOM is in fact an ARRAY reference filled with class names we'll send the meat_pack a hash of class names mapped to objects.
 
 =cut
 
@@ -204,10 +197,6 @@ sub new {
 # C<HTML::Tree> representation of the file via C<new> and then calls 
 # C<< $self->process($c, $c->stash) >> to rewrite the tree. 
 
-# NOTE WELL: most views store output in C<< $c->response->body >>. Seamstress
-# stores it in C<< $c->stash->{woven} >>.
-
-
 sub page2tree {
   my ($self, $c, $page_class, $process_method) = @_;
 
@@ -259,16 +248,9 @@ sub process {
 
     my ($skeleton, $meat, $body) ;
 
-    #warn "self_config(PROCESS): " . Dumper($self->config) ;
-    #warn "c_config(PROCESS): " . Dumper($c->config) ;
-
-
-    # 
-    # render $c->stash->{LOOM}
-    #
-
     my $loom = $c->stash->{LOOM};
 
+    # check we actually got a loom to  work with:
     unless ($loom) {
         $c->log->debug('No LOOM specified for rendering') if $c->debug;
         return 0;
@@ -280,27 +262,23 @@ sub process {
 
 
     if (ref($loom) eq 'ARRAY') {
-
-      map {
-	$meat->{$_} = $self->page2tree($c, $_);
-      } @$loom;
-
+        map {
+            $meat->{$_} = $self->page2tree($c, $_);
+        } @$loom;
     } else {
-
       $meat = $body = $self->page2tree($c, $loom);
-
     }
 
-    # 
+    #
     # render and pack MyApp::View::Seamstress->config->{skeleton}
     # if defined
     #
 
     if ($skeleton = $self->config->{skeleton}) {
       $skeleton = $self->page2tree($c, $skeleton);
-      #warn "SAH: ", $skeleton->as_HTML;
+
       $self->config->{meat_pack}->(
-	$self, $c, $c->stash, $meat, $skeleton
+          $self, $c, $c->stash, $meat, $skeleton
        );
 
       # $body = $self->page2tree($c, $skeleton, 'fixup');
@@ -309,15 +287,13 @@ sub process {
       $body = $skeleton ;
     }
 
-    #
-    #
-    #
-    $self->config->{fixup}->($body, $c);
+    # give the main view config an opportunity to twiddle the tree a bit:
+    if ( ref $self->config->{fixup} ) {
+        $self->config->{fixup}->($body, $c);
+    }
 
-    # 
-    # take the the body 
-    #
 
+    # take the the body and make some REAL html out of it!
     my $response_body;
     if ( $c->config->{use_xhtml} ) {
         $response_body = $body->as_XML( undef, ' ' );
@@ -325,12 +301,66 @@ sub process {
     else {
         $response_body = $body->as_HTML(undef, ' ')
     }
- 
+
+    # stuff the response_body in the response body!
     $c->response->body( $response_body );
 
+
+    # If loom is a class-name we're not doing anything especially fancy, so
+    # we'll just delete it:
+    unless ( ref $loom ) {
+        $body->delete;
+    }
+
     return 1;
 }
 
+;1;
+__END__
+
+=head1 The meat-skeleton paradigm!
+
+Generally Catalyst::View::Seamstress operates in one of 2 ways: a plain meat
+way or a meat-skeleton way.
+
+Plain meat is simple: the View takes C<<$c->stash->{LOOM} >> and calls
+C<new()> and C<process()> on it and stores the result in C<<$c->response->body>>.
+
+Meat-skeleton is designed to facilitate the way that most web sites are
+typically designed:
+
+HTML pages typically have meat and a skeleton. The meat varies from page
+to page while the skeleton is fairly (though not completely) 
+static. For example, the skeleton of a webpage is usually a header, a
+footer, and a navbar. The meat is what shows up when you click on a
+link on the page somewhere. While the meat will change with each
+click, the skeleton is rather static.
+
+
+Mason accomodates the meat-skeleton paradigm via
+an C<autohandler> and C<< $m->call_next() >>. Template 
+accomodates it via its C<WRAPPER> directive.
+
+And Seamstress? Well, here's what you _can_ do:
+
+=over
+
+=item 1 generate the meat, C<$meat>
+
+This is typically what you see in the C<body> part of an HTML page
+
+=item 2 generate the skeleton, C<$skeleton>
+
+This is typically the html, head, and maybe some body 
+
+=item 3 put the meat in the skeleton
+
+=back
+
+So, nothing about this is forced. This is just how I typically do
+things and that is why
+L<Catalyst::View::Seamstress|Catalyst::View::Seamstress> has support
+for this.
 
 =head1 Tips to View Writers
 
@@ -391,8 +421,8 @@ A working sample app, which does both simple and
 meat-skeleton rendering is available from
 Sourceforge CVS:
 
- cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/seamstress login
- cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/seamstress co -P catalyst-simpleapp
+ cvs -d:pserver:anonymous@seamstress.cvs.sourceforge.net:/cvsroot/seamstress login
+ cvs -z3 -d:pserver:anonymous@seamstress.cvs.sourceforge.net:/cvsroot/seamstress co -P catalyst-simpleapp
 
 =head1 SUPPORT
 
@@ -409,4 +439,4 @@ under the same terms as Perl itself.
 
 =cut
 
-1;
+
